% Author: Stilian Stoev <sstoev@math.bu.edu>
%
% This function performs DWT of given data sample via MRA, by using Daubechies 
% Wavelets. It invokes the subroutine "mydwt.m" to perform the one-step 
% multiresolution analysis.
%
% INPUT:
%
% x        <- data sample(s) (n x Nx) matrix. Each row contains a different 
%             sample.
% zmoments <- the number of zero moments (1 for Haar)
% scls     <- a vector of scales = powers of 2. It is required that they 
%                                 be sorted in ascending order
%
% outp     <- 1 to print progress info
% 
% varargin{1} <- if 1, then uses initialization (Copied from the code 
%                in "LDestimate" of Veitch and Abry).
%
% varargin{2} <- * if 1, then produces L2 normalized wavelet coefficients
%                * if 0 (default), then produces L1 normalized coefficients
%
% OUTPUT:
%
% D        <- (Nx1) cell of matrices, which contain the detail wavelet 
%            coefficients at each of the available scales. N = log_2(Nx),
%            where Nx=length(x).
% 
% Remarks:
%  (1) D{k}(i,:) = To get the vector of the k-(th) scale coefficients of the
%               i-(th) sample.
%  (2) D{1}      = are the finest level coefficients.
%
%  (3) The wavelet coefficients D{k} are L1-normalized (not L2 as in 
%       LDestimte of Abry and Veitch).
%  ********************************************************************    
%  *
%  *  SET varargin{2} = 1 if L2-normalized wavelet coefficients are 
%  *  needed.
%  *
%  *******************************************************************    
%
%  USAGE:
%
%  function [D,C] = MRA(x,zmoments,scls,outp,varargin{:})
%
%  Written by S. Stoev (sstoev@math.bu.edu).  Latest vestion: Feb, 6, 2004.
%
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the
%  authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
function [D,C] = MRA(x,zmoments,scls,outp,varargin);
    [n,Nx] = size(x);
        jj = fix(log2(scls)); % Find the octaves
       mjj = max(jj);
         m = length(jj);
         D = cell(1,m);
         C = cell(1,m);
    wttype = ['db' num2str(zmoments)];

if min([n,Nx])==1,
 if n>Nx,
  Nx = n;
  x  = x';
 end;
end;

if outp == 1,
 fprintf('\n Subroutine MRA working...\n')
end;

for i=1:n,
 if (outp == 1),
  fprintf('.');
 end;
 k=1; j=1;

if isempty(varargin),
  c=x(i,:);
elseif varargin{1} == 1,
 %
 % Initialization: copied from the code of Darryl Veitch
 % 
  lengthIwant = 0; %automatic choice of filter length  
  [c,k1,k2] = initDWT_discrete(x(i,:),zmoments,lengthIwant,0) ;
else
 c=x(i,:);  
end;

 [c, d] = mydwt(c,wttype);
 if (j==jj(k))&(~isempty(d)),
    if (length(varargin)>=2)&(varargin{2}),
     D{k}(i,:) = d; % USE L2 normalization
    else
     D{k}(i,:)=d*2^(-k*.5); % USE L1 normalization
    end;
    C{k}(i,:)=c;
    k=k+1;
  end;
 while ~isempty(d)&(j<mjj),
  j=j+1;
  [c, d] = mydwt(c,wttype);
  if (j==jj(k))&(~isempty(d)),
    if (length(varargin)>=2)&(varargin{2}),
     D{k}(i,:) = d; % USE L2 normalization
    else
     D{k}(i,:)=d*2^(-k*.5); % USE L1 normalization
    end;
    C{k}(i,:)=c;
    k=k+1;
  end;
 end;
end;

if outp==1,
 fprintf('\n MRA: Done. \n');
end;
