% Author: Stilian Stoev <sstoev@math.bu.edu>
%
% This function is ment to be a practical tool for LOCAL SELF-SIMILARITY 
% and LONG-RANGE DEPENDENCE analysis of a time-series.  It provides versatile
% sections of the DWT spectrum of the data and enables the practitioner to tell
% when there *might* be a CHANGE OF REGIME in the time series.  It was designed
% to aide the analysis of INTERNET TRAFFIC taces.
%
% INPUT:
%
%  x      <- the data (in stationary noise format)
%
%  par    <- a structured variable containing the parameters
%
%  par.tau     <- (default = 1) the time-duration of the aggregations of the data (in sec.).
%                 (used only for display purposes)
%
%  par.window  <- (default = max(fix(length(x)/20),1000)) the size of the window
%                 (in number of datapoints) for the local analysis
% 
%  par.j1      <- (default = 2) the lowest dyadic scale to be used to estimate the
%                 local self-similarity exponent H.
%
%  par.j2      <- (default = max available) the highest time scale used to estimate
%                 the local self-similarity exponent H.
%                 
%  par.zm      <- (default = 3) the number of zero moments for the Daubechies
%                  mother wavelet invloved in the analysis
%
% par.spectrum <- the type of spectrum to use:
%                 0 <= (default) Use the Abry-Veitch method
%                 1 <= Use the robustified form of the Abry-Veitch estimator.
%                 2 <= Use the Log-type estimator
%                 3 <= the Abry-Veitch method (on the motion time series)
%                 4 <= the robustified Abry-Veitch method (on the motion time series)  
%                 5 <= the Log-type estimator (on the motion time series)
%
% par.variance <- 
%                 0 <= use OLS (ord. least squares)
%                 1 <= (default) WLS
%                 2 <= use GLS (with the sample cov matrix)
%
%
% par.recmovie <- (default = 0) If 1, then records a movie of the analysis.
%
% par.topause  <- (default = 1) If 1, then pauses and waits for <Enter> to continue.
%
% par.init     <- (default = 1) Use initialization in Mallat's algorithm.
%                  0 do not use special initialization in Mallat's algorithm.
%
% par.cmap     <- (default = 1) Use colors.
%
% par.fsize    <- (default = 11) The fontsize to use.
%
% par.output   <- (default = 1) if 0, no graphical output.
% 
% OUTPUT:
%
%  M  <- contains a movie if recmovie ==1, otherwise is []
%  H  <- the time series of local estimates of H
%  SH <- the time series of the standard deviations of the estimates of H
%  mu <- the local mean (per window)
%  sd <- the local std (per window)
%  D  <- the local wavelet spectra
%
% USAGE:
%
%  [M,H,SH,mu,sd,D] = animate3(x,par);
%
% Written by sstoev@math.bu.edu, First version: 10.26.2003, Latest version: 1.28.2004
%  
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and 
%  "initfilterDaub_DWT_discrete.m" were borrowed from the package "LDestimate" written
%  by P. Abry and D. Veitch (see http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz),
%  with the kind permission of the authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
function [M,H,SH,mu,sd,D] = animate3(x,varargin);

[mx,nx] = size(x);

if min(mx,nx)>1,
       error('Data must be a vector!');
end;
if mx>nx,
    nx = mx;
    x = x';
end;

%
%
% Initializing parameters in "par"
%
%
DefaultPar = struct('tau',1,'window',max(fix(nx/20),1000),'j1',2,'j2',fix(log2(nx))+1,...
                    'zm',3,'spectrum',0,'regweight',1,'recmovie',0,'topause',1,'init',0,...
                    'cmap',1,'fsize',11,'output',1);

par = DefaultPar;
%
% Checking and assigning parameters
%
if length(varargin)>0,

 if isfield(varargin{1},'tau'), 
   par.tau = varargin{1}.tau;
 end; 
 if isfield(varargin{1},'window'), 
   par.window = varargin{1}.window;
 end; 
 if isfield(varargin{1},'j1'), 
   par.j1 = varargin{1}.j1;
 end; 
 if isfield(varargin{1},'j2'), 
   par.j2 = varargin{1}.j2;
 end; 
 if isfield(varargin{1},'zm'), 
   par.zm = varargin{1}.zm;
 end; 
 if isfield(varargin{1},'spectrum'), 
   par.spectrum = varargin{1}.spectrum;
 end; 
 if isfield(varargin{1},'regweight'), 
   par.regweight = varargin{1}.regweight;
 end;
 if isfield(varargin{1},'recmovie'), 
   par.recmovie = varargin{1}.recmovie;
 end;
 if isfield(varargin{1},'topause'), 
   par.topause = varargin{1}.topause;
 end;
 if isfield(varargin{1},'init'), 
   par.init = varargin{1}.init;
 end;
 if isfield(varargin{1},'cmap'), 
   par.cmap = varargin{1}.cmap;
 end; 
 if isfield(varargin{1},'fsize'), 
   par.fsize = varargin{1}.fsize;
 end; 
 
end;

fsize = par.fsize;
tau = par.tau;
windw = par.window;

%
%
%  Assigning colormaps
%
%

if par.cmap,
  cmap = 'default';
else
  cmap = ([0:255]')*[1,1,1]./255;
end;


%
%
% Computing local mean's and local std's
%
%
x = cumsum(x);

mu = diff(x(1:windw:nx));
sd = [];
for i = 1:length(mu),
  fprintf('.');
  sd = [sd, std(diff(x(1+(i-1)*windw:i*windw)))]; 
end;
%
% Computing the local spectra
%
fprintf('\n Computing the DWT spectra ... ');

[D, S, N] = dwtspec3(diff(x),par.window,par.zm,par.init,par.spectrum);

fprintf('done.\n');

[mD,nD] = size(D);

j1 = par.j1;
j2 = min(par.j2,mD);
j1 = min(j2-1,j1);

nN = length(N);
if nN>2,
  [maph,H,SH,CC,W] = sizeh3(D(j1:j2,:),S(j1:j2,:),N,0.05,par.regweight,0);
C = W*(D(j1:j2-1,:) -[j1:j2-1]'*H );
else
  H =[];
  fprintf('\n Not enough scales to estimate H.\n');
end;



if par.output,

if par.recmovie,
  M = moviein(nD);
else
   M= [];
end;

fighand = figure;
set(fighand,'Position',[ 296   204   600   480]);
clf;

for j=1:nD,
   
 subplot(2,2,1);
 hand = zeros(1,mD+2);
 hand(1) = plot(2*D(:,j)-[1:mD]','r'); hold on; 
 for i=1:mD,
     hand(i+1) = plot([i,i],[2*D(i,j)-i+2*1.96*S(i,j),...
                             2*D(i,j)-i-2*1.96*S(i,j)],'g');
 end;
  hand(mD+2) = plot([j1,mD],[(2*H(j)-1)*j1 + 2*C(j), (2*H(j)-1)*mD + 2*C(j)],'r');
  set(hand(mD+2),'LineWidth',1);
 if j==1,
     set(gca,'FontSize',fsize);
     title('DWT spectrum: regression');
     xlabel('dyadic scales');
     
 end;
 subplot(2,2,3);
 if j==1,
     imagesc(2*D-[1:mD]'*ones(1,nD));
     colormap(cmap);
     set(gca,'FontSize',fsize);
     title('DWT spectrum: features');
     xlabel(['time [1 unit = ', num2str(tau*windw),' sec]']);
     hold on;
 end;
 handsl(1) = plot([j-1,j-1],[1,mD],'r');
 handsl(2) = plot([j+1,j+1],[1,mD],'r');
 handsl(3) = plot([j-1,j+1],[mD,mD],'r');
 handsl(4) = plot([j-1,j+1],[1,1],'r');
 
 if j==1,
  figure(fighand);
  subplot(2,2,2);
  plot(H);
  set(gca,'FontSize',fsize);
  title(['Local H(t): j_1= ',num2str(j1)]);
  xlabel(['time [1 unit = ', num2str(tau*windw), ' sec]']);
  a = axis;
  axis([a(1:2), 0,1.2]);
  hold on;
  subplot(2,2,4);
  plot(mu);
  set(gca,'FontSize',fsize);
  title('Local mean');
  xlabel(['time [1 unit = ', num2str(tau*windw), ' sec]']);
  
  hold on;
  figure(fighand);
end;
 subplot(2,2,2);
 hhand = plot([j,j],[H(j)-1.96*SH(j),H(j)+1.96*SH(j)],'r');
 subplot(2,2,4);
 a = axis;
 h2(1) = plot([j-1,j-1],[a(3:4)],'r');
 h2(2) = plot([j+1,j+1],[a(3:4)],'r');
 h2(3) = plot([j-1,j+1],[a(4),a(4)],'r');
 h2(4) = plot([j-1,j+1],[a(3),a(3)],'r');
 


 figure(fighand);
 subplot(2,2,2);
 
 if par.recmovie,
    M(:,j) = getframe(fighand);
 end;
   
 if par.topause,
  pause;
 else
  pause(0.2);
 end;
 for ii = 1:mD+2,
     figure(fighand);
     delete(hand(ii));
 end;
 for ii = 1:4,
     figure(fighand);
     subplot(2,2,3);
     delete(handsl(ii));
     subplot(2,2,4);
     delete(h2(ii));
 end;
     delete(hhand);
 
 figure(fighand);
 subplot(2,2,1); plot(2*D(:,j)-[1:mD]','c');
 
end;

else
 M = [];
end;
