% Author: Stilian Stoev <sstoev@math.bu.edu>.
%  
% This function is part of the LASS tool package (see lass.m).
%
% This function computes matrices of *local wavelet spectra* by using the
% function "logdwtsp3.m".
%
% INPUT:
%
% x        <- in *stationary* noise format
% window   <- the window size
% zmoments <- number of zero moments of the Daubechies mother wavelet
% init     <- If 1, uses initialization (see the help of "logdwtsp"). 
% option   <- (= par.spectrum in the help of "lass.m")
%                       * 0 = Abry-Veitch (on the stationary time series)
% (STAT TIME SER)       * 1 = Robust Abry-Veitch (on the stationary time series)
%                       * 2 = "log"-type (on the stationary time series)
%
%                       * 3 = Abry-Veitch (on the motion time series)
% (MOTION w/ STAT INCR) * 4 = Robust Abry-Veitch (on the motion time series)
%                       * 5 = "log"-type  (on the motion time series)
%
%                
% OUTPUT:
%
%  D       <- the wavelet spectrum (see below and also the descr. of "option" above)
%  S       <- standard error of the stat in D
%
%  N       <- a vector of the numbers of available wavelet coefficients on
%             each scale.
%
% --------------------------------------------------------------------------------
% D2       <- a (m x K) matrix.  Columns yield local wavelet spectra, 
%             K = [N/window] denotes the number of windows, m - the maximum
%             number of available scales in the wavelet transform.
%             D2(j,k) corresponds to the log-mean-square spectrum y(j) of
%             function "logdwtsp3".
% S2       <- a (m x K)  matrix of sample estimates of the standad
%             deviations of the statistics in D2.
%
% Dlog     <- a (m x K) matrix, analogous to D2.  Corresponds to the
%             mean-log-square spectrum (output ylog(j) of function "logdwtsp").
% Slog     <- sample standard deviations of Dlog
%
% Drob     <- a (m x K) matrix, analogous to D2.  Corresponds to the
%             *robust* log-median-square spectrum (output yrob(j) of function "logdwtsp").
% Srob     <- sample standard deviations of Drob
%
%-----------------------------------------------------------------------------------------
% *NOTE:  The spectra D(j,k) are normalized so that D(j,k) ~ H*j + const
% (see also the remarks in the help of function "logdwtsp3".
%   
%
% USAGE:
%
% [D,S,N] = dwtspec3(x,window,zmoments,init,option);
%
% EXAMPLE OF USE:
%
% x = randn(1,100000);
% [D2,S2] = dwtspec2(x,5000,3,0,0);
% imagesc(D2);
%
%  Written by S. Stoev <sstoev@math.bu.edu>.  Latest vestion: Feb, 6, 2004.
%
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the
%  authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
    
function [D, S, N] = dwtspec3(x,window,zmoments,init,option);

% cast x in row-vector format
[mx,nx] = size(x);
if mx>nx,
 x = x';
 nx = mx;
end;

%
% Compute the window overlap
%
filt = rlistcoefdaub(zmoments);
olap = length(filt)*(fix(log2(window))+1);
clear('filt');
%

nw = fix(nx/window);
D = [];
S = [];
N = [];
 
for k = 1:nw,

 if (k~=1)&(k~=nw),
  [y,sy,N] = ...
  logdwtsp3(x(max([1,1+(k-1)*window-olap]):min([k*window+olap,nx])),zmoments,init,option);
 elseif k==1,
  [y,sy,N] = ...
   logdwtsp3(x(1+(k-1)*window:min([k*window+olap,nx])),zmoments,init,option);
 else
  [y, sy,N]= ...
   logdwtsp3(x(max([1,1+(k-1)*window-olap]):min([k*window+olap,nx])),zmoments,init,option);
 end;
 %
 % fprintf('.');
 %
 D = [D, y'];
 S = [S, sy'];  
end;
