%  Author: Stilian Stoev <sstoev@math.bu.edu>.
%
%  This is the main function for of the MATLAB Local Analysis of Self-Similarity tool (LASS).
%  The tool is described in details in the paper "LASS: A tool for the local analysis of 
%  self-similarity" by S. Stoev, M.S. Taqqu, C. Park, G. Michailidis and J.S. Marron (Preprint, 
%  2004).
%
%  It uses the personal functions:
%
%    dwtspec3.m (logdwtsp3, MRA.m, rlistcoefdaub.m, psi.m, initDWT_discrete.m,
%                initfilterDaub_DWT_discrete.m, robms.m)
%
%   sizeh3.m, scaleh3.m (dwtspec3.m) and formatfig.m.
%
%  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz).  S. Stoev would like to thank 
%  D. Veitch and P. Abry who kindly allowed us to include these functions in our LASS tool.
%
%  All other MATLAB scripts in this package were written by Stilian Stoev (sstoev@math.bu.edu).
%
%  DESCRIPTION OF THE INPUT AND OUTPUT PARAMETERS AND EXAMPLES OF USE
%
%  INPUT:
%
%  data   <- a vector of the data (stationary format)
%  
%  par    <- is a variable of type structure, which specifies the calling parameters, 
%            types of estimators, etc.
%
%  par.zmoments  <-     the number of zero moments for the mother Daubechies wavelet 
%                       (default = 3).
%
%  par.initDWT   <-     * 0 = does not pre-filter in the initialization of the Mallat's
%                         algorithm
%                       * 1 = (default) uses the initialization of Veitch, Taqqu and Abry.
%
%  par.spectrum  <- 
%                       * 0 = (default) Abry-Veitch (on the stationary time series)
% (STAT TIME SER)       * 1 = Robust Abry-Veitch (on the stationary time series)
%                       * 2 = "log"-type (on the stationary time series)
%
%                       * 3 = Abry-Veitch (on the motion time series)
% (MOTION w/ STAT INCR) * 4 = Robust Abry-Veitch (on the motion time series)
%                       * 5 = "log"-type  (on the motion time series)
%
%  par.weights  <- 
%                       * 0 = OLS (ordinary least squares)
%                       * 1 = (default) WLS (weighted least squares)
%                       * 2 = GLS (with the sample cov marix if possible WLS used otherwise
%  
%  par.variance <- 
%                       * 0 = (default) assume independence of the statistics 
%                         H(j,k) = S(j+1,k) - S(j,k), where eg in the case of
%                         Abry-Veitch spectrum: S(j,k) = log2(mean{d(j,k)^2,k=1,...,Nj})/2+0.5*j
%                         Consequently, model Var(H(j,k)) by Var(S(j+1,k))-Var(S(j,k)).
%
%                       * 1 = use the sample cov. matrix of the vectors 
%                         (H(j,k),j=j1,...,j2-1)
%
%  par.window   <-      size of the window (default = min(n,max(fix(n/20),2000))).
%
%  par.alevel   <-      the level of significance (default = 0.05, i.e. 95%)
%                       
%  par.windows  <-      a vector of window sizes (default = fix(par.window*[0.5, 1]))
%
%  par.j1       <-      starting scale (default = 1)
% 
%  par.loop     <-      * 1 = (default) loop to determine best (j1)
%                       * 0 = do not loop.
%
%  par.output   <-      * 0 = no output whatsoever
%                       * 1 = (default) graphical output
%                       * 2 = no graphical output (outputs results, see below)
%                       * 3 = graphical output and results (see below)
%
%  par.report   <-      * 0 = do not print anything while working
%                       * 1 = (default) print report and progress information while running
%  
%  par.cmap     <-      * 0 = use monochrome mode for the color diagrams
%                       * 1 = (default) use "SiZer color codes" for the color diagrams 
%                  
%  par.save     <-      * 0 = (default) do not save the figures as "fig" files
%                       * 1 = save the figures as "fig" files
%  
%  par.psprint  <-      * 0 = (default) do NOT save the figures as (color) "ps" files
%                       * 1 = save the figures as (color) "ps" files
%
%  par.extreme  <-      * 0 = do not show Step 5 (Visualization of extreme fluctuations in H(r))
%                       * 1 = (default) show Step 5 (Visualization of extreme fluctuations in H(r)) 
%
%  par.overlay  <-      * 0 =  plots the mean of the local Hurst estimates in Step 4
%                       * 1 =  (default) "overlays" on the same plot the local Hurst est. from Step4
%
%  OUTPUT:
%
%   results     <- a structured variable with the results (the fields of results are described below)
% 
%   <fields of results>:  
%
%     H1        <-  the initial estimate of local H(k)
%     SH1       <-  the sample standard deviations of H1 
%     Firstj1   <-  the value of "j1" for the initial estimate of local H(k) 
%     MapH1     <-  the map of significant deviations from linear spectra
%     CC1       <-  the sample covariance matrix of the statistics (H(.,k)) 
%     
%     H2        <-  the secondary estimate of local H(k)  
%     SH2       <-  the sample standard deviations of H2
%     Secondj1  <-  the value of "j1"  used for the initial estimate of local H(k) 
%     MapH2     <-  the map of significant deviations from linear spectra
%     CC2       <-  the sample covariance matrix of the statistics (H(.,k)) 
%
%     j2        <-  the value of "j2" used in the local estimates H(k)
%
%     H         <-  a cell-array of the local estimates of H for different windows.  
%                   H{i} corresponds to par.windows(i).
%     SH        <-  a cell-array of the sample standard deviations corresponding to H above. 
%     MapH      <-  a matrix of the "interpolated" local H(k) from H above
%     MapPval   <-  a matrix of the P-values associated with deviations from stationarity in 
%                   the rows of "MapH"
%
%  USAGE:
%    (1) a simple call: 
%     >> lass(data);
%
%    (2) a customized call: 
%     >> R = lass(data,struct('window',20000,'windows',5000*[1:10]));
%
%  Written by S. Stoev <sstoev@math.bu.edu>.  Latest vestion: Feb, 6, 2004.
%
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
function  results = lass(data,varargin);

 [m,n] = size(data);
 
 if min(m,n)>1,
     error('The variable <data> must be a vector, not a matrix.');
 elseif n<m,
     data = data';
     n = m;
 end;
 
 if n<2000,
     error('The dataset must be at least 2000 long!');
 end;
 

  window = min(n,max(fix(n/20),2000));
  
  DefaultPar = struct('zmoments',3,'spectrum',0,'weights',1,'variance',0,...
           'window',window,'windows',fix(window*[0.5,1])+1,'j1',1,...
           'loop',1,'output',1,'alevel',0.05,'initDWT',1,'report',1,...
           'cmap',1,'save',0,'psprint',0,'extreme',1,'overlay',1);
 
 if length(varargin)<1,   
     par = DefaultPar;
 else
      
     par = DefaultPar;
     
     if isfield(varargin{1},'zmoments'),
        par.zmoments = varargin{1}.zmoments;
     end;
     if isfield(varargin{1},'spectrum'),
        par.spectrum = varargin{1}.spectrum;
     end;
     if isfield(varargin{1},'weights'),
        par.weights = varargin{1}.weights;
     end;
     if isfield(varargin{1},'variance'),
        par.variance = varargin{1}.variance;
     end;
     if isfield(varargin{1},'window'),
        par.window = varargin{1}.window;
     end;
     if isfield(varargin{1},'windows'),
        par.windows = varargin{1}.windows;
     end;
     if isfield(varargin{1},'j1'),
        par.j1 = varargin{1}.j1;
     end;
     if isfield(varargin{1},'loop'),
        par.loop = varargin{1}.loop;
     end;
     if isfield(varargin{1},'output'),
        par.output = varargin{1}.output;
     end;
     if isfield(varargin{1},'alevel'),
        par.alevel = varargin{1}.alevel;
     end;
     if isfield(varargin{1},'initDWT'),
        par.initDWT = varargin{1}.initDWT;
     end;
     if isfield(varargin{1},'report'),
        par.report = varargin{1}.report;
     end;
     if isfield(varargin{1},'cmap'),
        par.cmap = varargin{1}.cmap;
     end;
     if isfield(varargin{1},'save'),
        par.save = varargin{1}.save;
     end;
     if isfield(varargin{1},'psprint'),
        par.psprint = varargin{1}.psprint;
     end;
     if isfield(varargin{1},'extreme'),
        par.extreme = varargin{1}.extreme;
     end;
     if isfield(varargin{1},'overlay'),
        par.overlay = varargin{1}.overlay;
     end;
    
     warning('No validity check for "par" is implemented yet.  Use with caution!');   
 end;

 %
 % Set the color map for Steps 1, 2 and 4
 % 
  if par.cmap,
    cmapFlag = 1;
    par.cmap = [[1,0,0];[0.5,0,0.5];[0,0,1]];
  else
    cmapFlag = 0;
    par.cmap = [[1,1,1];[0.5,0.5,0.5];[0,0,0]];
  end;
%
%
% Results:
%
%

    results = struct('H1',[],'SH1',[],'Firstj1',[],'MapH1',[],'CC1',[],...
                     'H2',[],'SH2',[],'Secondj1',[],'MapH2',[],'CC2',[],...
                     'j2',[],'H',[],'SH',[],'MapH',[],'MapPval',[]);

%
% Report:
%
if par.report,
     fprintf('\n * Computing with parameters: \n');

fprintf('\t Spectrum type (0 - A-V; 1 - Robust A-V; 2 - Log-type) + 3 (if motion): %d \n ',par.spectrum);
fprintf('\t Variance assumptions (0 - independence; 1 - sample cov; \n');
fprintf('\t                      *2 - post smooth* not implemented yet*) ........: %d \n ',par.variance);
fprintf('\t Regression weights   (0 - OLS; 1 - WLS; 2 - GLS) ....................: %d \n ',par.weights);
fprintf('\t Initialization       (0 - raw sample; 1 - Veitch,Taqqu,Abry .........: %d \n ',par.initDWT);
fprintf('\t Zero moments         .... ...........................................: %d \n ',par.zmoments);
fprintf('\t Level of significance ...............................................: %1.2f  \n',par.alevel);

fprintf('\n * Sample and window sizs: \n');
fprintf('\t Sample size ........................: %d  \n',length(data));
fprintf('\t Initial window size ................: %d  \n',par.window);
fprintf('\t Multiple window sizes ..............: \n '); fprintf('| %d |',par.windows); 
fprintf('\n \n'); 
   
warning on;
 
else
warning off;
end;

 % Step 1
 if par.report,
   fprintf(' * STEPs 1 and 2 ( Visualization of the local wavelet spectra ) \n ');
 end;

 motion = 0; % this is the "motion" flag
 if par.spectrum>=3,
     motion = 1;
 end;
 
 %
 % Compute the appropriate wavelet spectrum
 %

 if par.report,
   fprintf('\t Computing wavelet spectra ... '); 
 end;

 option = par.spectrum;
 [D, S, N] = dwtspec3(data,par.window,par.zmoments,par.initDWT,option);

  if par.report,
   fprintf('done. \n'); 
  end;


 [mD,nD] = size(D);
 
  %
 %  Step 1:
 %
 j1 = min(par.j1,mD-1);
 j1 = max(j1,1);
 j2 = mD;

 
 [maph1,H1,SH1,CC1] = sizeh3(D(j1:mD,:),S(j1:mD,:),N(j1:mD),par.alevel,par.weights,par.variance);


 results.H1 = H1;
 results.SH1 = SH1;
 results.CC1 = CC1;
 results.Firstj1 = j1;
 results.MapH1 = maph1;

 Firstj1 = j1; 

 %
 %  Plot the Figure:
 %
 if par.output,
     figs = [];
 currentFig = figure; 
 figs = [figs, currentFig];
 
     figure(currentFig);
     subplot(2,1,1);
     hold off;
     plot(H1);
     hold on;
     plot([1,length(H1)],[mean(H1),mean(H1)],':k');
     legend('Estimated H(r)',['Mean = ', num2str(mean(H1))]);
     plot(H1+1.96*SH1,'--r');
     plot(H1-1.96*SH1,'--r');
    
     axis tight;
     a = axis;
     axis([a(1:2), 0.45,1.05]);
     hold off;
     
     subplot(2,1,2);
     hold off;
     imagesc(maph1,[-1,1]);
     colormap(par.cmap);

     formatfig(1,currentFig,par,j1,j2);
 end;
 %
 %  End Plotting the Figure. 
 %
 
 if (par.loop)&(par.output),
 %
 % Step 2:
 %
 
    H2 = H1;
   SH2 = SH1;
 maph2 = maph1;
   CC2 = CC1;

 flag = 1;
 while 1,
   
     j1new = input(' Enter a new value for j1 = ');
     if isempty(j1new),
         break;
     else
         j1 = j1new;
     end;
     
     j1 = min(mD-1,j1);
     j1 = max(j1,1);
     
     [maph2,H2,SH2,CC2] = sizeh3(D(j1:mD,:),S(j1:mD,:),N(j1:mD),par.alevel,par.weights,par.variance);

     if flag,
         flag = 0;
         currentFig = figure; 
         figs = [figs, currentFig];
     end;
 
     figure(currentFig);
     subplot(2,1,1);
     hold off;
     plot(H2);
     hold on;
     plot([1,length(H2)],[mean(H2),mean(H2)],':k');
     legend('Estimated H(r)',['Mean = ', num2str(mean(H2))]); 
     plot(H2+1.96*SH2,'--r');
     plot(H2-1.96*SH2,'--r');
     axis tight;
     a = axis;
     axis([a(1:2), 0.45,1.05]);
     hold off;
     
     subplot(2,1,2);
     hold off;
     imagesc(maph2,[-1,1]);

set(gca,'YTickMode','auto');
set(gca,'YTickLabelMode','auto');
set(gca,'YTicklabel',[j1:j2-1]);
set(gca,'YTick',[Firstj1:j2-1]);

     colormap(par.cmap);

     formatfig(2,currentFig,par,j1,j2);
 end

results.H2 = H2;
results.SH2 = SH2;
results.Secondj1 = j1;
results.MapH2 = maph2;
results.CC2 = CC2;
results.j2 = j2;

 end;

%
% Step 3
%

if par.loop,
    currentFig = figure;
    figs = [figs, currentFig];
    
    subplot(2,1,1);
    plot(H1);
    hold on;
    plot(H2,'--r');
    legend(['H(k) scales (j1,j2) = (' num2str(Firstj1),',',num2str(j2),')'],...
    ['H(k) scales (j1,j2) = (' num2str(j1),',',num2str(j2),')']);
    
    mdata =[];
    sdata =[];
    k = fix(n/par.window);
    for i = 1:k-1,
        mdata = [mdata, mean(data((i-1)*par.window+1:i*par.window))];
        sdata = [sdata, std(data((i-1)*par.window+1:i*par.window))];
    end;
    mdata = [mdata, mean(data((k-1)*par.window+1:n))];
    sdata = [sdata, std(data((k-1)*par.window+1:n))];
    
    subplot(2,2,3);
    plot(mdata);
    
    subplot(2,2,4);
    plot(sdata);

    formatfig(3,currentFig,par,j1,j2);
end;


 
%
% Step 4
%


if par.report,
  fprintf(' * STEP 4 ( Influence of the window size ) \n'); 
end;


if par.output,
    nooutput = 0;
    monocm = 1;
else
    monocm = 0;
    nooutput = 1;
end;

    newfig = 0;

j2 = fix(log2(max(par.windows)))+2;

if par.report,
  fprintf('\t Computing local wavelet spectra: '); 
end;

[MapH,H,MapPval,MapSH,SH,G] = ...
scaleh3(data, par.zmoments, par.windows, j1, j2, par.alevel, par.spectrum, par.weights, ...
              par.variance, par.initDWT, monocm, newfig, 1);


results.H = H;
results.SH = SH;
results.MapH = MapH;
results.MapPval = MapPval;

%
%
%  STEP 4 (begin)
%
%

if par.output,

  currentFig = figure;
  figs = [figs, currentFig];
 
subplot(2,1,1);

if ~cmapFlag,
 cmh = fliplr([0.5:0.1:1]);
 cm = cmh'*[1,1,1];
 colormap(cm);
end;

 imagesc(MapH,[0.45,1.05]);



subplot(2,1,2);

if par.overlay,
 for i = 1:length(H),
   phand = plot(MapH(i,:),'b');
   set(phand,'LineWidth',1*i/length(H));
   hold on;
 end;
 phand = plot(mean(MapH),'k');
 set(phand,'LineWidth',2);
else
 mMapH = mean(MapH);
 sMapH = std(MapH);

 plot(mMapH);
 hold on;
 
 plot(mMapH+1.96*sMapH,':g');
 plot(mMapH-1.96*sMapH,':g');
end;

 axis tight;
 a = axis;
 axis([a(1:2), 0.45, 1.05]);


formatfig(4,currentFig,par,j1,j2);


if par.extreme,

%
%
% STEP 5
%
%

if par.report,
  fprintf('\n * STEP 5 ( Visualizing extreme fluctuations in H(r) ) \n'); 
end;

currentFig = figure;
figs = [figs, currentFig];

subplot(2,1,2);

quant = norminv(1-par.alevel/2);

for i = 1:length(par.windows),
 mh = mean(H{i});
%
% sh = std(H{i});
 sh = SH{i}(1);
 plot([i-0.2,i+0.2],[mh,mh],'b');
 hold on;
 plot([i-0.15,i+0.15],[mh+quant*sh,mh+quant*sh],'r');
 plot([i-0.15,i+0.15],[mh-quant*sh,mh-quant*sh],'r');
 plot([i,i],[mh-quant*sh,mh+quant*sh],':g');
end;

subplot(2,1,1);

j = find(MapPval>1-par.alevel/2);

MapP = zeros(size(MapPval));
MapP(j) = 1;

j = find(MapPval<par.alevel/2);
MapP(j) = -1;

imagesc(MapP,[-1,1]);
hold on;

formatfig(5,currentFig,par,j1,j2);

end;
end;
     
%
%
% SAVE FIGURES
% 
%
if par.save,

if par.report,
 fprintf('\n Saving figures as MATLAB FIG-files ...');
end;

for i = 1 :length(figs),
 figure(figs(i));
 fname = get(figs(i),'FileName');
 hgsave(fname);
end;

if par.report,
 fprintf(' done.');
end;
end;

if par.psprint,

if par.report,
 fprintf('\n Saving figures as PS-files ...');
end;

for i = 1 :length(figs),
 figure(figs(i));
 fname = get(figs(i),'FileName');
 if cmapFlag,
  print('-dpsc',fname);
 else
  print('-dps',fname);
 end; 
end;

if par.report,
 fprintf(' done.');
end;

end;

%
%
% DONE.  GOOD BYE!
% 
%
     
if par.report,
 fprintf(' done. \n'); 
 fprintf('\n Thank you for using the LASS tool! \n');
 fprintf('\n Please email (sstoev@math.bu.edu) if you have any comments or suggestions.\n\n');
end;
