% Author: Stilian Stoev <sstoev@math.bu.edu>.
%
% This function computes the Log-DWT-spectrum of a given time series and is part of the
% LASS tool (see lass.m) but it may be also used independently.
%
% It uses personal functions  MRA.m, rlistcoefdaub.m, psi.m, initDWT_discrete.m,
%  initfilterDaub_DWT_discrete.m and robms.m.
%
% DESCRIPTION OF THE INPUT AND OUTPUT PARAMETERS AND THE USE OF THE FUNCTION
%
% INPUT:
%
%  x        <- the data (in *stationary* noise format)
%
%  zmoments <- number of zero moments for the Daubechies mother wavelet.
%
%  init     <- If 1, then initializes the Mallat's Fast DWT algorithm
%          using the method in Veitch, Taqqu and Abry "Meaningful MRA
%          initialisation for discrete time series", Signal Processing, 
%          2000, Vol. 80, pp. 1971-1983. (Code was borrowed from D.
%          Veitch and P. Abry's package for "LDestimate.m" see 
%          http://www.emulab.ee.mu.oz.au/~darryl/).
%
%  option   <- 
%                       * 0 = (default) Abry-Veitch (on the stationary time series)
% (STAT TIME SER)       * 1 = Robust Abry-Veitch (on the stationary time series)
%                       * 2 = "log"-type (on the stationary time series)
%                       * 3 = Abry-Veitch (on the motion time series)
% (MOTION w/ STAT INCR) * 4 = Robust Abry-Veitch (on the motion time series)
%                       * 5 = "log"-type  (on the motion time series)
%
% OUTPUT:
%
%  y    <- a wavelet spectrum (see the description below)
%  sy   <- est. sample variance of the stat. in y (see descr. below)
%
%  N     <- a vector with the number of wavelet coefficients on each scale
%           used to obtain the statistics y* and s*.
%
%  ---------------------------------------------------------------------------
%
%  yraw <- the *raw* log-mean-squares of the wavelet coefficients along scales
%
%  y    <- yraw + bias correction due to non-linearity of the Log-function
%          (Used is a first-order Taylor series expansion and CLT 
%           approximation similar to that in the package "LDestimate.m" of
%           Veitch and Abry.)
% 
%  sy   <- sample estimates of the standard deviations of the statistics
%          in the vector y.
%
%  yav  <- Abry-Veitch log-spectrum (= yraw + bias correction as computed 
%          in the package "LDestimate" of Veitch and Abry)
%
%  savy <- estimate of the *theoretical* standard deviations of the statistics 
%          in yav (as computed in the package "LDestimate" of Abry and Veitch)   
%
%  ylog  <- mean-log-absolute values of the wavelet coefficients along
%          scales.  This, log-type spectrum, is theoretically "unbiased".
%          See, for example, Pipiras, Taqqu and Abry, Asymptotic normality for 
%          wavelet-based estimators of fractional stable motion (2001, Preprint) 
%          and Stoev, Pipiras and Taqqu, "Estimation of the self-similarity 
%          parameter in linear fractional stable motion", Signal
%          Processing, 2002, Vol. 82, pp. 1873-1901.
%          
%  sylog <- sample estimates of the standard deviations of the statistics
%          in the vector ylog.
%
%  yrob  <- log-median-square of the wavelet coefficients along scales.
%           This median-type wavelet spectrum is robust to single large
%           fluctuations in the wavelet coefficients due to
%           non-stationarity. *EXPERIMENTAL* See Stoev, Taqqu, Park, Michailidis 
%           and Marron (2004), LASS: a tool for the local analysis of self-similarity
%           in Internet traffic, (2004, Preprint).
%
%  syrob <- sample estimates of the standard deviations of the statistics
%          in the vector yrob. *EXPERIMENTAL*
%
% *NOTES:
%       (1) The wavelet spectra yraw(j), y(j), yav(j), ylog(j) and yrob(j) are normalized so
%         that
%         y(j) ~ H*j + const
%         and hence the "slope" yelds estimates of the Hurst parameter H.
% 
%       (2) To get the Log-scale Diagram of Abry & Veitch (as in
%       LDestimate) one should set: S(j) := 2*y(j)-j, where y = {yraw, y,
%       yav, ylog, yrob}.
%         
% USAGE:
%
%  [y,sy,N] = logdwtsp3(x,zmoments,init,option);
%
%
%  Written by S. Stoev <sstoev@math.bu.edu>.  Latest vestion: Feb, 6, 2004.
%
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the
%  authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
function [y,sy,N] = logdwtsp3(x,zmoments,init,option);

     % Cast x into a row-vector format.     
     [mx,nx] = size(x);
     if mx>nx,
       nx = mx;
       x = x';
     end;
 
 if option>2,
     motion =1;
     x = cumsum(x);
     option = option -3;
 else
     motion =0;
 end;
 
    j2 = fix(log2(nx))-1; %set j2 = max scale
     D = MRA(x,zmoments,2.^[1:j2],0,init,1);
     m = length(D);
     
     y = [];
     sy =[];
     N = [];
     
 
switch option,
case {0}, % Abry - Veitch spectrum
    
%
%Copied form Abry-Veitch (begin)
%
precis_psi  = 1e-4 ;
precis_zeta = 1e-4 ;
%
%Copied from Abry-Veitch (end)
%
     for i=1:m,
       nD = length(D{i});
       if nD>=5,
        N = [N, nD];
        md = mean(D{i}.^2);
%
%Copied from Abry-Veitch (begin)
%
  y  = [y, (0.5-motion)*i+(log2(mean(D{i}.^2))-psi(nD/2,precis_psi)*log2(exp(1))+log2(nD/2))/2];
  sy = [sy, log2(exp(1)) * sqrt(rzeta(fix(nD/2),precis_zeta))/2];        
%
%Copied from Abry-Veitch (end)
%  
       end;
     end;
   
%
case {1}, % Robust Abry-Veitch
%
for i=1:m,
       nD = length(D{i});
       if nD>=5, 
        [mdrob,sdrob] = robms(D{i}.^2);
        N = [N, nD];
        y =  [y , mdrob/2 + (0.5-motion)*i];
        sy = [sy, sdrob/2];
       end;
end;
    %
case {2}, % "log"-type spectrum
%   
 for i=1:m,
       nD = length(D{i});
   if nD>=5,
       N = [N, nD];
       y = [y, mean(log2(abs(D{i})))+(0.5-motion)*i];
       sy = [sy, std(log2(abs(D{i})))/sqrt(nD)];
   end;
 end;
 
otherwise,
    error('Unsupported type of spectrum, option = 0,1,2,3,4 or 5.');
end; %switch option
