%  Author: Stilian Stoev <sstoev@math.bu.edu>
%
%  This function is part of the LASS tool package (see lass.m).
%
%  Produces a "Scale Space Type" view of the estimated local self-similarity
%  parameter.  Namely, for a collection of window values given in the vector
%  "windows", the local scaling exponent H(t) is estimated using scales 
% "j1" and "j2".  The results are combined in the matrix "MapH" which is 
%  then displayed as an image.
%   
%  INPUT: 
%
%    x        <- data (in stationary noise format) 
%    zmoments <- number of zero moments for the Daubechies mother wavelet.
%    windows  <- a vector of window values
%    j1, j2   <- the first and last dyadic scales to use in the estimation
%                 of H(t).
%    alevel   <- the level of significance
%    option   <-  
%                       * 0 = (default) Abry-Veitch (on the stationary time series)
% (STAT TIME SER)       * 1 = Robust Abry-Veitch (on the stationary time series)
%                       * 2 = "log"-type (on the stationary time series)
%
%                       * 3 = Abry-Veitch (on the motion time series)
% (MOTION w/ STAT INCR) * 4 = Robust Abry-Veitch (on the motion time series)
%                       * 5 = "log"-type  (on the motion time series)
%
%
%    weight  <-  * If "0" then OLS regression is performed to estimate H(t)
%
%                * If "1" then WLS regression is used.  The weights 
%                  corresponding to the DWT spectra statistics are inversely
%                  proportional to their sample variances.
%
%                * If "2" then GLS regression is used.  he covariance matrix 
%                  of the errors is eastimated by using a sample covariance
%                  matrix, computed from the sample of DWT spectra in different
%                  windows (for more information, see, Stoev, Pipiras and
%                  Taqqu (2002), "Estimation of the self-similarity 
%                  parameter in linear fractional stable motion", Signal
%                  Processing, 2002, Vol. 82, pp. 1873-1901.).  If the sample size
%                  is not enough then WLS is used instead. 
%
%  variance <- 
%               * 0 = the assumption for *independence* of the statistics 
%                  H(j,k) = D(j+1,k) - D(j,k) is used to
%                  compute the sample variance of the estimators H(k)
%               * 1 = the sample covariance matrix of
%                  (H(j,k)), j=j1,...,j2-1 (over k) is used to compute the
%                  sample standard deviation of the estimates H(k).
%
%  init     <-  * 0 = does not use special initialization for the Mallat's algorithm
%               * 1 = uses the initialization due to Veitch, Taqqu and Abry.
%
%    monocm  <- * If "0", then multiple colors are used (with the standard
%                 colormap).
%               * If "1", then a "monochrome" colormap is used.
%
%    newfig  <- * If "1", then the output is drawn in a new figure
%               * If "0", then the current figure is used to draw.
%               
%   nooutput <- * If "1" then nothing is plotted.
%
% OUTPUT:
%
%    MapH    <- (nw x N) matrix, with the map of estimated H(t) for different
%               windows;  i(th) row of "MapH" corresponds to the i(th) window 
%               value "windows(i)".
%
%    MapSH   <- (nw x N) matrix, with the map of estimated standard deviations 
%               of H(t) for different windows;
%   
%    H       <- (1 x nw) array of cells with the estimated H(t); 
%               H{i} corresponds to H(t) for the i(th) window value.
%
%    SH      <- (1 x nw) array of cells with estimates of the sample standard
%               deviations corresponding to the estimates in H.
%
%    G       <- (1 x nw) array of cells with the estimated sample covariance
%               matrices in case weight = 2.  G = {} if weight = 0 or 1 was used. 
%
% USAGE:
%
%  [MapH,H,MapSH, SH,G] = ...
%  scaleh3(x,zmoments,windows,j1,j2,alevel,option,weight,variance,init,monocm,newfig,nooutput);
%
% EXAMPLE OF USE:
%
%  [MapH,H,SH,G] = ...
%  scaleh(randn(1,1000000),3,2000*[1:10],2,13,0,2,0,1);
%
%  Written by Stilian Stoev <sstoev@math.bu.edu>, Nov 10, 2003.
% 
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  (used in dwtspec3.m) were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.
%
function [MapH,H,MapPval,MapSH,SH,G] = ...
 scaleh3(x,zmoments,windows,j1,j2,alevel,option,weight,variance,init,monocm,newfig,nooutput);


 nw = length(windows);

 G = {};

 H = {};
 SH = {};

if (j2-j1<1),
      error('Not enough scales to perform estimation.');
  H = [H, {}];
 SH = [SH, {}];
end;

for k = 1:nw,
 
 [D, S, N] = dwtspec3(x,windows(k),zmoments,init,option);
 [mD,nD] = size(D);

 jj2 = min([mD,j2]);
 jj1 = min([mD,j1]);
if (mD<2)|(jj2-jj1<1),
  warning(['Not enough scales to perform estimation.  Window ignored...']);
else

dd = diff(D);
ss = sqrt(max(S(2:mD,:).^2-S(1:mD-1,:).^2,S(1:mD-1,:).^2));

GG = real((cov(dd(jj1:jj2-1,:)'))^(1/2));

 switch weight,
  case {0},
   w = (1/(jj2-jj1))*ones(jj2-jj1,nD);
 case {1},
   %w = 1./ss(jj1:jj2-1,:);
   w = sqrt(N(jj1:jj2-1)')*ones(1,nD);
   w = w./(ones(jj2-jj1,1)*sum(w));
 case {2},
   if (nD>=(min([5*mD,5]))),    
    Ginv = pinv(GG);
    X = ones(jj2-jj1,1);
    w = Ginv*pinv((Ginv*X)'*(Ginv*X))*(Ginv*X);
    A = w;
    w = w*ones(1,nD);
    G = [ G, {GG}];
   else
   %
   % If not enough windows - use the weights in case {1}.
   %
    GG = [];
    w = sqrt(N(jj1:jj2-1)')*ones(1,nD);
    w = w./(ones(jj2-jj1,1)*sum(w));
   end;
 end;

 H  = [H, {sum(dd(jj1:jj2-1,:).*w)}];
 
 if (variance)&(~isempty(GG)),
  SH = [SH, {sqrt(sum((GG*w).^2))}];
 else
  SH = [SH, {sqrt(sum(ss(jj1:jj2-1,:).*w).^2)}];
 end;
 end;
 fprintf('.');
end;

%
%  Constrct the Map
% 

ngrid = 1;
for k = 1:nw,
  nH=length(H{k});
  ngrid = max([ngrid,nH]);
end;

MapH = [];
MapSH = [];

for k = 1:nw,
 nH = length(H{k});
 if nH>1,
  jw = fix(ngrid/nH);
  mh = [];  
  msh = [];
  for ih = 1:nH-1,
      mh = [mh, ones(1,jw)*H{k}(ih)]; 
     msh = [msh, ones(1,jw)*SH{k}(ih)]; 
  end;
  mh = [mh, ones(1,ngrid-(nH-1)*jw)*H{k}(nH)];
  msh = [msh, ones(1,ngrid-(nH-1)*jw)*SH{k}(nH)];

  MapH = [MapH; mh];  

  MapSH = [MapSH; msh];  
 end;
end;

MapPval = (MapH - mean(MapH')'*ones(1,ngrid))./MapSH;
MapPval = erf(MapPval/sqrt(2))/2+0.5;

if ~nooutput,

if (newfig)&(~nooutput),
  figure;
end;

subplot(2,1,1);
plot(mean(MapH),'b');
hold on;
plot(mean(MapH)+1.96*std(MapH),':g');
plot(mean(MapH)-1.96*std(MapH),':g');

ylabel('H(t)');
title('Estimated average H(t) accross windows.');

subplot(2,1,2);

if monocm,
 cmh = fliplr([0.5:0.1:1]);
 cm = cmh'*[1,1,1];
 colormap(cm);
 imagesc(MapH,[0.45,1.05]);
else
 imagesc(MapH,[0.45,1.05]);
end;
colorbar('vert');

if ~nooutput,
 figure;
end;

subplot(2,1,1);

for i = 1:length(windows),
 mh = mean(H{i});
 sh = std(H{i});
 plot([i-0.2,i+0.2],[mh,mh],'b');
 hold on;
 plot([i-0.15,i+0.15],[mh+1.96*sh,mh+1.96*sh],'r');
 plot([i-0.15,i+0.15],[mh-1.96*sh,mh-1.96*sh],'r');
 plot([i,i],[mh-1.96*sh,mh+1.96*sh],':g');

end;
title('Confidence intervals for mean of H(t) per window.');
xlabel('Window');
ylabel('H');

subplot(2,1,2);
j = find(MapPval>1-alevel/2);

MapP = zeros(size(MapPval));
MapP(j) = 1;

j = find(MapPval<alevel/2);
MapP(j) = -1;

imagesc(MapP,[-1,1]);
colormap([[1 1 1];[.5,.5,.5];[0,0,0]]);
colorbar('vert');
end;
