% Author: Stilian Stoev <sstoev@math.bu.edu>
%
% This function is part of the LASS tool package (see lass.m).
%
% Estimates H(t) by using weighted least squares (WLS) regression from a matrix of
% wavelet spectra D by using the formula H = v'*diff(D), where
% "v" is a vector of weights. 
%
% Based on these estimates of H it constructs a SiZer-like map
%  * red  = +1 corresponds to significant increase in H at that scale
%  * blue = -1 corresponds to significant (1-alevel)*100% (default is 95%)
%                decrease of H at that scale
%  * green = 0 means no significant change in H
% 
% INPUT:
%
%  D      <- a (m x n) matrix of local Log-DWT spectra (as output by
%            "dwtspec2")
%  S      <- a (m x n) matrix of estimates of the standard deviations of the 
%             statistics in D (as output by "dwtspec2").
%
%  alevel <- the alpha-level of significance (if [], then 0.05) is used.
%
%  weight <- * 0 = uses OLS(ordinary least squares regr. to compute) H
%            
%            * 1 =  the estimates of H(t) are computed by using
%            "weighted" averages of the slopes, with weights
%             proportional to the square-root of the number of wavelet 
%             coefficients available at the corr. scale.
%              
%            * 2 = and if option == 1 the generalized least squares 
%             (GLS) regression with the sample covariance matrix is used to
%             estimate the local Hurst parameter.
%
%
%  option <- * if 0, then the *slopes* are assumed independent with variances
%            max(S^2(i+1)-S^2(i),S^2(i)/2).
% 
%            * if 1 (default), then the *slopes* are not assumed to be 
%            independent.  Their covariance matrix is estimated by using
%            the sample covariance matrix for the column-vectors of the
%            matrix D.
%
% OUTPUT:
%
%  maph <- a (m-1 x n) SiZer - like map (see above)
%
%  H    <- the estimated H(k), k=1,...,n
%
%  SH   <- standard deviations of the estimated H(t)
%  
%  CC   <- the sample covariance matrix of the vectors (H(j,k)),j=1,...,m-1
%          where H(j,k) = D(j+1,k) - D(j,k),
%
%  W    <- the vector of weights used in the regression
%
% USAGE:
%
%   [maph,H,SH,CC,W] = sizeh3(D,S,N,alevel,weight,option);
%
%  Written by Stilian Stoev <sstoev@math.bu.edu>, Nov 10, 2003.
% 
% * ACKNOWLEDGMENTS:
%
% *  The MATLAB functions "rlistcoefdaub.m", "psi.m", "initDWT_discrete.m", and "initfilterDaub_DWT_discrete.m"
%  (used in dwtspec3.m) were borrowed from the package "LDestimate" written by P. Abry and D. Veitch (see
%  http://www.cubinlab.ee.mu.oz.au/~darryl/LDestimate_code.tar.gz), with the kind permission of the authors.
%
% *  The code was designed while M. Taqqu and S. Stoev were visiting SAMSI (Statistical and Applied 
% Mathematical Sciences Institute) (http://www.samsi.info/) during the program of Network Modeling
% for the Internet in the Fall, 2003.  
%
function [maph,H,SH,CC,wei] = sizeh3(D,S,N,alevel,weight,option);

 epsilon = 0.000000000001;
 
 [m,n] = size(S);
 dd    = diff(D);

 if isempty(alevel),
   alevel = 0.05;
 end; 

 aquant = norminv(1-alevel/2);
 
 CC = cov(dd');
 
 switch weight,
 case {0},
     w = ones(m-1,n);
     wei = ones(1,m-1)./(m-1);
 case {1},
 
     w = (sqrt(N(1:m-1))'*ones(1,n));
     wei = sqrt(N(1:m-1))./sum(sqrt(N(1:m-1)));

 case {2},
  if (n>=5),    
    C = real(CC)^(1/2);
    Ginv = pinv(C);
    X = ones(m-1,1);
    w = Ginv*pinv((Ginv*X)'*(Ginv*X))*(Ginv*X);
    wei = w'/sum(w);
    w = w*ones(1,n);
  else
     CC=[];
     error('Not enough windows to compute the sample covariance matrix.  Try weight = 1.');
  end;
 end;
  w = w./(ones(m-1,1)*sum(w));

if m>=2,
 H = sum(dd.*w);
else 
 H = dd.*w;
end; 


switch option,
%    
case {0}, % assume independence
%
ss = sqrt(max(S(2:m,:).^2-S(1:m-1,:).^2, S(1:m-1,:).^2));

if m>=2,
 SH = sqrt(sum((ss.*w).^2));
else
 SH = abs(ss.*w);
end;

 W = [];
 w0 = w;
 for j = 1:m-1,
   w0(j,:) = w(j,:)-ones(1,n);
   if m>=2,
     W = [W; sqrt(sum((w0.*ss).^2))]; 
   else
     W = [W; abs(w0.*ss)];
   end;
 end;
 CC = [];
%
case {1}, % use sample cov matrix
%
  if (n>5),       
    C = real((CC)^(1/2));
  else
    CC = [];
    error('Not enough windows to compute covariances use option = 0 instead.');
  end;

 if m >=2,
  SH = sqrt(sum((C*w).^2));
 else
  SH = abs(C*w);
 end;

 W = [];
 w0 = w;
 for j = 1:m-1,
   w0(j,:) = w(j,:)-ones(1,n);
   if m >=2,
    W = [W; sqrt(sum((C*w0).^2))]; 
   else
    W = [W; abs(C*w0)]; 
   end;
 end;
end;

 ddp = dd - ones(m-1,1)*H - aquant*W;
 ddn = dd - ones(m-1,1)*H + aquant*W;

maph = zeros(size(dd));
 j = find(ddp>0);
maph(j) = 1;
 j = find(ddn<0);
maph(j) = -1;
